@@ -110,7 +110,7 @@ struct _GtkIconInfo
   /* Information about the source
    */
   gchar *filename;
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
   /* System codepage version of filename, for DLL ABI backward
    * compatibility functions.
    */
@@ -1322,7 +1322,7 @@ choose_icon (GtkIconTheme       *icon_theme,
 	icon_info->filename = g_strdup (unthemed_icon->svg_filename);
       else if (unthemed_icon->no_svg_filename)
 	icon_info->filename = g_strdup (unthemed_icon->no_svg_filename);
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
       icon_info->cp_filename = g_locale_from_utf8 (icon_info->filename,
 						   -1, NULL, NULL, NULL);
 #endif
@@ -2208,7 +2208,7 @@ theme_lookup_icon (IconTheme          *theme,
           file = g_strconcat (icon_name, string_from_suffix (suffix), NULL);
           icon_info->filename = g_build_filename (min_dir->dir, file, NULL);
           g_free (file);
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
           icon_info->cp_filename = g_locale_from_utf8 (icon_info->filename,
 						   -1, NULL, NULL, NULL);
 #endif
@@ -2216,7 +2216,7 @@ theme_lookup_icon (IconTheme          *theme,
       else
         {
           icon_info->filename = NULL;
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
           icon_info->cp_filename = NULL;
 #endif
         }
@@ -2674,7 +2674,7 @@ gtk_icon_info_free (GtkIconInfo *icon_info)
     return;
  
   g_free (icon_info->filename);
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
   g_free (icon_info->cp_filename);
 #endif
   if (icon_info->loadable)
@@ -2776,9 +2776,8 @@ static gboolean icon_info_ensure_scale_and_pixbuf (GtkIconInfo*, gboolean);
 static void 
 apply_emblems (GtkIconInfo *info)
 {
-  gint pos, w, h, ew, eh, x, y;
-  gdouble scale;
-  GdkPixbuf *icon, *emblem;
+  GdkPixbuf *icon;
+  gint w, h, pos;
   GSList *l;
 
   icon = info->pixbuf;
@@ -2788,41 +2787,48 @@ apply_emblems (GtkIconInfo *info)
   for (l = info->emblem_infos, pos = 0; l; l = l->next, pos++)
     {
       GtkIconInfo *emblem_info = l->data;
-      if (icon_info_ensure_scale_and_pixbuf (emblem_info, FALSE)) 
+
+      if (icon_info_ensure_scale_and_pixbuf (emblem_info, FALSE))
         {
-           emblem = emblem_info->pixbuf;
-           ew = gdk_pixbuf_get_width (emblem);
-           eh = gdk_pixbuf_get_height (emblem);
-           if (ew >= w)
-             {
-               scale = 0.75;
-               ew = ew * 0.75;
-               eh = eh * 0.75;
-             }
-           else 
-             scale = 1.0;
-           switch (pos % 4) 
-             {
-             case 0: 
-               x = w - ew;
-               y = h - eh;
-               break;
-             case 1: 
-               x = w - ew;
-               y = 0;
-               break;
-             case 2: 
-               x = 0;
-               y = h - eh;
-               break;
-             case 3: 
-               x = 0;
-               y = 0;
-               break;
-           }
-          gdk_pixbuf_composite (emblem, icon, x, y, ew, eh, x, y, 
+          GdkPixbuf *emblem = emblem_info->pixbuf;
+          gint ew, eh;
+          gint x = 0, y = 0; /* silence compiler */
+          gdouble scale;
+
+          ew = gdk_pixbuf_get_width (emblem);
+          eh = gdk_pixbuf_get_height (emblem);
+          if (ew >= w)
+            {
+              scale = 0.75;
+              ew = ew * 0.75;
+              eh = eh * 0.75;
+            }
+          else
+            scale = 1.0;
+
+          switch (pos % 4)
+            {
+            case 0:
+              x = w - ew;
+              y = h - eh;
+              break;
+            case 1:
+              x = w - ew;
+              y = 0;
+              break;
+            case 2:
+              x = 0;
+              y = h - eh;
+              break;
+            case 3:
+              x = 0;
+              y = 0;
+              break;
+            }
+
+          gdk_pixbuf_composite (emblem, icon, x, y, ew, eh, x, y,
                                 scale, scale, GDK_INTERP_BILINEAR, 255);
-       } 
+       }
    }
 }
 
@@ -3509,7 +3515,7 @@ gtk_icon_info_new_for_pixbuf (GtkIconTheme *icon_theme,
   return info;
 }
 
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32) && !defined (_WIN64)
 
 /* DLL ABI stability backward compatibility versions */
 
